import { SqliteStatement } from './sqlite-statement';
export interface SqliteModelOptions<Q extends string> {
    /** File where the DB is stored */
    dbPath: string;
    /**
     * Mode to use when opening the database
     * Can be one or more of `sqlite3.OPEN_READONLY`, `sqlite3.OPEN_READWRITE` and `sqlite3.OPEN_CREATE`
     * Default is `OPEN_READWRITE | OPEN_CREATE`
     */
    dbMode?: number;
    /**
     * If the database doesn't exist, initialize it with the provided SQL
     * Can be directly a list of SQL strings or file paths containing valid SQL
     */
    createDbSql: string[];
    /** List of queries to prepare. They will be available as `SqliteStatement` objects in `this.stmt` */
    queries: {
        [K in Q]: string;
    };
    /** sqlite3 verbose mode */
    verbose?: boolean;
    /** Name of the table to use for internal management */
    internalTable?: string;
}
export declare class SqliteModel<Q extends string> {
    protected readonly modelOptions: SqliteModelOptions<Q>;
    protected readonly stmt: {
        [K in Q]: SqliteStatement;
    };
    private readonly ready;
    private db;
    constructor(options: SqliteModelOptions<Q>);
    /**
     * Return a promise to resolve when the database is ready (or reject if any error)
     *
     * @example
     * const model = new Model(options);
     * await model.isReady();
     * model.foobar();
     */
    isReady(): Promise<void>;
    /**
     * Retrieve the current schema version
     */
    getCurrentSchemaVersion(): Promise<number>;
    /**
     * Closes the connection to the database and resolves when done
     */
    closeDb(): Promise<void>;
    /**
     * Read SQL code to execute from a file without the comments, to avoid runtime problems
     * and execute it, resolving or rejecting the returning promise when it finishes
     *
     * @param sql SQL Query string to execute
     */
    protected execSql(sql: string): Promise<void>;
    /**
     * Return SQL code to execute from a file without the comments, to avoid runtime problems
     *
     * @param file Path to the file with SQL code
     */
    protected getSqlFromFile(file: string): Promise<string>;
    /**
     * Prepare the provided sql query as a promisify wrapper around Statement
     */
    protected prepareStmt(sql: string): Promise<SqliteStatement>;
    /**
     * Open the database (or create it if it doesn't exist) and resolve when ready
     */
    private openDb;
    /**
     * Check if the model hasn't been initializated yet
     */
    private isNew;
    /**
     * Create the internal table if it doesn't exist
     */
    private createInternalTable;
    /**
     * Create the model tables in the database from the specified files or sql
     */
    private createModelTables;
    /**
     * Prepare all the model queries as a promisified queries in `this.stmt`
     */
    private prepareStmts;
}
