"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sqlite3 = require("sqlite3");
const fs_1 = require("fs");
const mkdirp_1 = require("mkdirp");
const path_1 = require("path");
const async_secuential_1 = require("./async-secuential");
const async_parallel_1 = require("./async-parallel");
const sqlite_statement_1 = require("./sqlite-statement");
class SqliteModel {
    constructor(options) {
        this.modelOptions = Object.assign({ dbMode: sqlite3.OPEN_READWRITE | sqlite3.OPEN_CREATE, verbose: false, internalTable: '_model' }, options);
        this.stmt = {};
        const { verbose } = this.modelOptions;
        if (verbose) {
            sqlite3.verbose();
        }
        this.ready = this.openDb().then(() => { });
    }
    /**
     * Return a promise to resolve when the database is ready (or reject if any error)
     *
     * @example
     * const model = new Model(options);
     * await model.isReady();
     * model.foobar();
     */
    isReady() {
        return this.ready;
    }
    /**
     * Retrieve the current schema version
     */
    getCurrentSchemaVersion() {
        return this.ready.then(() => new Promise((resolve, reject) => {
            const getVersionSql = `SELECT value FROM ${this.modelOptions.internalTable} WHERE key = ?`;
            this.db.get(getVersionSql, ['version'], (error, row) => {
                if (error) {
                    reject(error);
                    return;
                }
                const currentVersion = row && Number(row.value);
                resolve(currentVersion);
            });
        }));
    }
    /**
     * Closes the connection to the database and resolves when done
     */
    closeDb() {
        return this.ready.then(() => new Promise((resolve, reject) => {
            this.db.close((error) => {
                if (error) {
                    reject(`Error closing the database: ${error}`);
                    return;
                }
                resolve();
            });
        }));
    }
    /**
     * Read SQL code to execute from a file without the comments, to avoid runtime problems
     * and execute it, resolving or rejecting the returning promise when it finishes
     *
     * @param sql SQL Query string to execute
     */
    execSql(sql) {
        return new Promise((resolve, reject) => {
            this.db.exec(sql, function cb(error) {
                if (error) {
                    reject(`Error while executing sql: ${sql} (${error})`);
                    return;
                }
                resolve();
            });
        });
    }
    /**
     * Return SQL code to execute from a file without the comments, to avoid runtime problems
     *
     * @param file Path to the file with SQL code
     */
    getSqlFromFile(file) {
        return new Promise((resolve, reject) => {
            let sql;
            try {
                sql = fs_1.readFileSync(file)
                    .toString()
                    .replace(/-- .*\n/gm, '');
            }
            catch (error) {
                reject(`Error while reading sql from ${file} (${error})`);
                return;
            }
            resolve(sql);
        });
    }
    /**
     * Prepare the provided sql query as a promisify wrapper around Statement
     */
    prepareStmt(sql) {
        return new Promise((resolve, reject) => {
            this.db.prepare(sql, function prepared(error) {
                if (error) {
                    reject(`Error preparing query ${sql} (${error})`);
                    return;
                }
                const stmt = {
                    bind: sqlite_statement_1.promisifyStatementResult(this, 'bind'),
                    reset: sqlite_statement_1.promisifyStatementResult(this, 'reset'),
                    finalize: sqlite_statement_1.promisifyStatementResult(this, 'finalize'),
                    run: sqlite_statement_1.promisifyStatementResult(this, 'run'),
                    get: sqlite_statement_1.promisifyStatementResult(this, 'get', 'row'),
                    all: sqlite_statement_1.promisifyStatementResult(this, 'all', 'rows'),
                    each: sqlite_statement_1.promisifyStatementEach(this),
                };
                resolve(stmt);
            });
        });
    }
    /**
     * Open the database (or create it if it doesn't exist) and resolve when ready
     */
    openDb() {
        return new Promise((resolve, reject) => {
            const { dbPath, dbMode } = this.modelOptions;
            mkdirp_1.sync(path_1.dirname(dbPath));
            this.db = new sqlite3.Database(dbPath, dbMode, async (error) => {
                if (error) {
                    reject(`sqlite: error opening the database: ${error}`);
                    return;
                }
                if (await this.isNew()) {
                    await this.createInternalTable();
                    await this.createModelTables();
                }
                await this.prepareStmts();
                resolve();
            });
        });
    }
    /**
     * Check if the model hasn't been initializated yet
     */
    isNew() {
        return new Promise((resolve, reject) => {
            const { internalTable } = this.modelOptions;
            const checkSql = `SELECT name FROM sqlite_master WHERE type='table' AND name='${internalTable}'`;
            this.db.get(checkSql, (error, row) => {
                if (error) {
                    reject(`Error while checking for internal table (${error})`);
                    return;
                }
                resolve(!row);
            });
        });
    }
    /**
     * Create the internal table if it doesn't exist
     */
    createInternalTable() {
        return new Promise((resolve, reject) => {
            const { internalTable } = this.modelOptions;
            const sql = `
        CREATE TABLE IF NOT EXISTS ${internalTable} (
          key text NOT NULL PRIMARY KEY,
          value text NOT NULL
        );

        INSERT INTO ${internalTable} VALUES('version', '1');
      `;
            this.db.exec(sql, (error) => {
                if (error) {
                    reject(`Error while creating internal table (${error})`);
                    return;
                }
                resolve();
            });
        });
    }
    /**
     * Create the model tables in the database from the specified files or sql
     */
    async createModelTables() {
        await async_secuential_1.asyncSecuential(this.modelOptions.createDbSql, async (fileOrSql) => {
            if (fs_1.existsSync(fileOrSql)) {
                const sql = await this.getSqlFromFile(fileOrSql);
                return this.execSql(sql);
            }
            return this.execSql(fileOrSql);
        });
    }
    /**
     * Prepare all the model queries as a promisified queries in `this.stmt`
     */
    async prepareStmts() {
        const { queries } = this.modelOptions;
        const stmt = this.stmt;
        await async_parallel_1.asyncParallel(Object.keys(queries), async (query) => {
            stmt[query] = await this.prepareStmt(queries[query]);
        });
    }
}
exports.SqliteModel = SqliteModel;
